import re
import logging
import subprocess

logger = logging.getLogger(__name__)


def run(cmd, exit_status = 0, stdout_regexp = None, stderr_regexp = None, **kwargs) -> bool:
    # subprocess.CompletedProcess
    res =  subprocess.run(
        cmd.split(' '), stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )

    logger.debug(logger)
    if not res.returncode == exit_status:
        return False
    elif stdout_regexp and not re.findall(stdout_regexp, res.stdout.decode()):
        return False
    elif stderr_regexp and not re.findall(stderr_regexp, res.stderr.decode()):
        return False

    return True




import logging
import operator as op
import psutil

logger = logging.getLogger(__name__)


def memory_conv(res, unit, **kwargs) -> int:
    if unit.lower() == "gb":
        res = ((res / 1024) / 1024) / 1024
    elif unit.lower() == "mb":
        res =  (res / 1024) / 1024
    elif unit.lower() == "kb":
        res = res / 1024
    return round(res, 2)


def cores(operator='ge', value=0, **kwargs) -> int:
    """
        see https://docs.python.org/3/library/operator.html
    """
    cores = psutil.cpu_count()
    logger.debug(f"Found {cores} CPU cores")
    return getattr(op, operator)(cores, value)


def ram(unit="MB", kind='free', operator='ge', value=0, **kwargs) -> int:
    """
        kind = [
             'active',
             'available',
             'buffers',
             'cached',
             'count',
             'free',
             'inactive',
             'index',
             'percent',
             'shared',
             'slab',
             'total',
             'used'
        ]
    """
    # total physical memory available usually in Bytes
    res = getattr(psutil.virtual_memory(), kind)
    memory = memory_conv(res, unit)
    logger.debug(f"Found {memory}{unit} {kind} RAM")
    return getattr(op, operator)(memory, value)





import logging
import psutil
import socket

LOCAL_ADDRESS = [
    '127.0.0.1',
    '0.0.0.0',
    '::'
]
logger = logging.getLogger(__name__)


def ingoing_port(port, status='LISTEN', kind='tcp', addrs=LOCAL_ADDRESS, **kwargs) -> bool:
    """
    +------------+----------------------------------------------------+
    | Kind Value | Connections using                                  |
    +------------+----------------------------------------------------+
    | inet       | IPv4 and IPv6                                      |
    | inet4      | IPv4                                               |
    | inet6      | IPv6                                               |
    | tcp        | TCP                                                |
    | tcp4       | TCP over IPv4                                      |
    | tcp6       | TCP over IPv6                                      |
    | udp        | UDP                                                |
    | udp4       | UDP over IPv4                                      |
    | udp6       | UDP over IPv6                                      |
    | unix       | UNIX socket (both UDP and TCP protocols)           |
    | all        | the sum of all the possible families and protocols |
    +------------+----------------------------------------------------+
    """
    for i in psutil.net_connections(kind=kind):
        logger.debug(i)
        result = all(
            (
                i.status == status,
                i.laddr.ip in addrs,
                i.laddr.port == port
            )
        )
        if result:
            return result
        else:
            result = False

    if not result and status in ['CLOSED', 'NA']:
        return True
    return result


def outgoing_port(addr, port, timeout=4, kind=None, **kwargs) -> bool:
   # TODO: kind is unused !!!
   s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
   s.settimeout(timeout)
   try:
      s.connect((addr, int(port)))
      s.shutdown(2)
      return True
   except:
      return False



import importlib
import logging

from copy import deepcopy

logger = logging.getLogger(__name__)


class CheckMaster:
    def __init__(self, conf, log_style = 'label', tags=None):
        self.conf = conf
        self.result = deepcopy(conf)
        self.tags = tags
        self.log_style = log_style

    def load_plugin(self, plugin_name):
        n1, _, n2 = plugin_name.rpartition(".")
        module = importlib.import_module(n1)
        func = getattr(module, n2)
        return func

    def _check(self, func_name, func, rule, tags):
        if self.tags and rule.get('tag') not in tags:
            return

        _check = func(**rule)
        if self.log_style == 'label':
            msg = f"{rule.get('label') or rule}"
        else:
            msg = f"{func_name} {rule}"
        if not _check:
            logger.error(msg)
        else:
            logger.info(msg)

    def start(self):
        for _func, rules in self.result.items():
            func = self.load_plugin(_func)
            if isinstance(rules, dict):
                self._check(_func, func, rules, self.tags)
            elif isinstance(rules, list):
                for rule in rules:
                    self._check(_func, func, rule, self.tags)
            else:
                raise NotImplemented(_func)



import distro as dist
import logging
import platform as pf

from multiprocessing import cpu_count
from . status import get_distro_info

logger = logging.getLogger(__name__)


def architecture(kind, **kwargs) -> str:
    """example: ('64bit', 'ELF')"""
    return kind == pf.architecture()

def processor(kind, **kwargs) -> str:
    """example: x86_64"""
    return kind == pf.processor()

def platform(kind, **kwargs) -> str:
    """Linux-5.4.0-88-generic-x86_64-with-glibc2.29"""
    return kind == pf.platform()

def system(kind, **kwargs) -> str:
    """example: Linux"""
    return kind == pf.system()

def linux_kernel(kind, **kwargs) -> str:
    """example: 5.4.0-88-generic"""
    return kind == pf.release()

def system(kind, **kwargs) -> str:
    """example: Linux"""
    return kind == pf.system()

def distro(**kwargs) -> dict:
    """example: Linux"""
    values = get_distro_info()

    for i in kwargs:
        if i not in values:
            logger.warning(f'{i} is not a valid distribution attribute')
        if kwargs[i].lower() != values[i].lower():
            return False
    return True



import logging
import operator as op
import os
import psutil

from datetime import datetime

from checkmaster.hardware import memory_conv

logger = logging.getLogger(__name__)


def paths(
        path, kind='file', status = "present",
        permissions = None, uid = None, gid = None, **kwargs
    ):
    if kind == 'file':
         _func = os.path.isfile
    elif kind == 'directory':
        _func = os.path.isdir

    try:
        exists = _func(path)
    except FileNotFoundError as e:
        if status != 'present':
            return True
        else:
            return False

    # existence
    if exists and status == 'present':
        status = True
    elif not exists and status == 'absent':
        return True
    elif not exists and status == 'present':
        return False
    else:
        raise NotImplemented()

    # permissions
    if permissions or uid or gid:
        try:
            perms = get_permissions(path)
            logger.debug(perms)
        except FileNotFoundError as e:
            logger.error(e)
            return False
    else:
        return status

    # TODO: additional checks on atime, ctime, mtime are possible
    if permissions and permissions != perms['permissions']:
        return False
    elif uid and uid != perms['user']:
        return False
    elif gid and gid != perms['group']:
        return False
    else:
        return status


def get_permissions(path) -> dict:
    """example: 0o100664"""
    st = os.stat(path)
    perm = str(oct(st.st_mode))
    return {
        'raw_permissions': perm,
        'permissions': perm[4:],
        'user': st.st_uid,
        'group': st.st_gid,
        'create': datetime.fromtimestamp(st.st_ctime),
        'modified': datetime.fromtimestamp(st.st_mtime),
        'last_access': datetime.fromtimestamp(st.st_atime)
    }


def size(path, unit='MB', kind='free', operator='ge', value=100, **kwargs) -> int:
    """
        kind = set(
            total, used, free, percent
        )
    """
    size = psutil.disk_usage(path)
    if kind == 'percent':
        size = size.percent
    else:
        size =  memory_conv(getattr(size, kind), unit)
    value = float(value)
    logger.debug(f"Found {size}{unit} in {path}")
    return getattr(op, operator)(size, value)



import distro as dist
import logging
import psutil
import requests
import socket

from . hardware import memory_conv

logger = logging.getLogger(__name__)


def get_ips(url="https://icanhazip.com/"):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(4)
    s.connect(("8.8.8.8", 80))
    _ip = s.getsockname()[0]
    _hostn = socket.gethostbyaddr(_ip)

    ips = {
        'public ip': requests.get(url).content.decode().strip(),
        'private ip': _ip,
        'private hostname': _hostn[0],
        'other private ips': print(", ".join(_hostn[1] if len(_hostn)>1 else []))
    }
    return ips


def get_distro_info():
    values = {
        'base': dist.like(),
        'name': dist.id(),
        'codename': dist.codename(),
        'version': dist.version(),
        'cores' : psutil.cpu_count(),
        'free_ram' : memory_conv(psutil.virtual_memory().free, unit='gb')
    }
    return values


def sockets_processes_names():
    res = {}

    for i in psutil.net_connections():
        _proc = psutil.Process(i.pid)
        _name = _proc.name()
        logger.debug(_proc)
        data = dict(
            pid = _proc.pid,
            exe = _proc.exe(),
            process_name = _name,
            port = i.laddr.port,
            bind = i.laddr.ip
        )
        if _name in res:
            res[_name].append(data)
        else:
            res[_name] = [data]
    return res
